/*

  Test program to measure Fl_Image_Surface::image() speed.

  Creates a large image, draws some things on it
  and then measures the time it takes to read the image.

  fltk-config --use-images --compile image_surface_read_image_speed.cxx

*/

static const int WIDTH = 20000;
static const int HEIGHT = 1200;

#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Image_Surface.H>
#include <FL/Fl_RGB_Image.H>
#include <FL/Fl_Scroll.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Box.H>
#include <FL/fl_draw.H>
#include <cstdlib>
#include <cassert>
#include <ctime>
#include <cmath>

#include <sys/time.h>
#include <unistd.h>
#include <iostream>

using namespace std;

static Fl_Image *getImage( int w, int h )
{
	Fl_Image_Surface *img_surf = new Fl_Image_Surface(w, h); // create an Fl_Image_Surface object
	assert( img_surf );

	img_surf->set_current(); // direct graphics requests to the image

	srand( time( 0 ) );
	fl_color(FL_WHITE); fl_rectf(0, 0, w, h); // draw a white background
	int n = w / 10;

	for ( int i = 0; i < n; i++ )
	{
		// draw filled circles
		int x = random() % w;
		int y = random() % h;
		int r = random() % h + 20;

		fl_color( fl_rgb_color( random() % 256, random() % 256, random() % 256) );
		fl_pie( x, y, r, r, 0., 360 );
	}

	// get the image once...
	// This obviously seems to to trigger additional, internal
	// tasks (like filling some caches of X, or flushing the graphics)
	// and takes notably longer
	cout << "First call Fl_Image_Surface::image()" << endl;
	Fl_RGB_Image *image = img_surf->image();
	delete image; // ..and discard it immediately

	// do now the real measurement
	cout << "Second call Fl_Image_Surface::image()" << endl;
	timeval start, end;
	gettimeofday( &start, 0 );
	/*Fl_RGB_Image **/image = img_surf->image();        // get the resulting image
	gettimeofday( &end, 0 );
	cout << "Time Fl_Image_Surface::image(): " << lround( 0.001 * ((end.tv_sec-start.tv_sec) * 1000000 + (end.tv_usec-start.tv_usec)) ) << " ms" << endl << endl;

	delete img_surf;                                // delete the img_surf object

	Fl_Display_Device::display_device()->set_current(); // direct graphics requests back to the display

	return image;
} // getImage

static void cb_redo( Fl_Widget *w, void *d_ )
{
	Fl_Window *win = (Fl_Window *)d_;
	Fl_Box *b = (Fl_Box *)(((Fl_Scroll *)win->child(0))->child(0));
	Fl_Image *image = b->image();
	b->image( 0 );
	delete image;
	b->image( getImage( WIDTH, HEIGHT ) );
	win->redraw();
}

int main()
{
	Fl_Double_Window win( 800, 600, "Test Fl_Image_Surface::image() speed" );
	Fl_Scroll s( 0, 0, win.w(), win.h() );
	Fl_Box b( 0, 0, WIDTH, HEIGHT );
	b.align( FL_ALIGN_IMAGE_BACKDROP );
	Fl_Button but( 150, 100, 70, 40, "redo" );
	s.end();
	but.callback( cb_redo, &win );
	win.end();
	win.resizable( s );
	win.show();
	b.image( getImage( WIDTH, HEIGHT ) );
	return Fl::run();
}
